/*
 * Copyright(c) 2007 Intel Corporation. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * Maintained at www.Open-FCoE.org
 */
#ifndef _FCOEIOCTL_H_
#define _FCOEIOCTL_H_

#include <linux/ioctl.h>
#include <linux/types.h>


/*
 * ioctl part of the driver structure
 */

/* fcoe_type / mode flags */
#define INITIATOR_MODE		0x01
#define TARGET_MODE 		0x02
#define FCOE_CFG_TYPE_T11 	0x04	/* fcoe_type for T11 protocol */

#ifndef ETH_ALEN
#define ETH_ALEN   6
#endif /* ETH_ALEN */

#ifndef IFNAMSIZ
#define IFNAMSIZ   16
#endif /* IFNAMSIZ */


struct fcoe_cfg {
	fc_wwn_t	wwnn;
	fc_wwn_t	wwpn;
	char		local_ifname[IFNAMSIZ];
	u_int8_t	interface_type;
	u_int8_t	fcoe_type;
	u_int8_t	intr_delay_timer_enable;
	u_int8_t	intr_delay_timer_def_value;
	u_int8_t	login_timeout;
	u_int8_t	login_retry_count;
	u_int8_t	login_down_time;
	u_int8_t	resv0;
	u_int32_t	fcoe_mtu;
	u_int8_t	dest_addr[ETH_ALEN];
	u_int8_t	src_addr[ETH_ALEN];
	u_int8_t	vlan_tag;
	u_int8_t 	rsv[3];
} __attribute__((aligned (8)));


#define FCOE_IOCTL_MAGIC  0xfc
#define FCOE_CREATE_INTERFACE	_IOW(FCOE_IOCTL_MAGIC, 1, struct fcoe_cfg)
#define FCOE_DESTROY_INTERFACE  _IOW(FCOE_IOCTL_MAGIC, 2, struct fcoe_cfg)
#define FCOE_IOCTL_VERSION	1
#define FCOE_MAXNR  2

#endif /* _FCOEIOCTL_H_ */
