/*
 * Copyright (c) 2008 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef mlx4_fc_bitmap_H
#define mlx4_fc_bitmap_H

struct mlx4_fc_bitmap {
	unsigned long *addr;
	unsigned size;
	unsigned long last_bit;
};

static inline struct mlx4_fc_bitmap *mlx4_fc_bitmap_alloc(unsigned size)
{
	struct mlx4_fc_bitmap *bitmap;

	
	if (!(bitmap = kzalloc(sizeof(struct mlx4_fc_bitmap), GFP_KERNEL))) 
		goto alloc_error;

	if (!(bitmap->addr = kzalloc(sizeof(unsigned long) * BITS_TO_LONGS(size), GFP_KERNEL)))
		goto alloc_error;
	
	bitmap->size = size;
	
	return bitmap;

alloc_error:
	printk("Error allocating memory\n");
	return NULL;
}

static inline void mlx4_fc_bitmap_free(struct mlx4_fc_bitmap *bitmap)
{
	kfree(bitmap->addr);
	kfree(bitmap);
}

static inline int mlx4_fc_bitmap_slot_alloc(struct mlx4_fc_bitmap *bm)
{
	int slot_num;

	do {
		slot_num = find_next_zero_bit(bm->addr, bm->size, (bm->last_bit + 1) % bm->size);
		if (slot_num >= bm->size) {
			slot_num = find_first_zero_bit(bm->addr, bm->size);
			if (slot_num >= bm->size) {
				printk("Couldn't find a free slot\n");
				return -1;
			}
		}
	} while (test_and_set_bit(slot_num, bm->addr));

	bm->last_bit = slot_num;
	return slot_num;
}

static inline void mlx4_fc_bitmap_slot_free(struct mlx4_fc_bitmap *bm, int slot_num)
{
	if (slot_num >= bm->size) {
		printk("Error: Trying to free out of bound slot number\n");
	}
	clear_bit(slot_num, bm->addr);
}

#endif

