/*
 * Copyright (c) 2008 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef MLX4_FC_H
#define MLX4_FC_H

#include <linux/compiler.h>
#include <linux/list.h>
#include <linux/mutex.h>
#include <linux/skbuff.h>
#include <linux/if_ether.h>
#include <scsi/scsi_cmnd.h>

#include <linux/mlx4/device.h>
#include <linux/mlx4/qp.h>
#include <linux/mlx4/doorbell.h>
#include <linux/mlx4/qp.h>
#include <linux/mlx4/srq.h>
#include <linux/mlx4/cq.h>

#include "mlx4_fc_dbg.h"
#include "fc_types.h"
#include "fc_frame.h"
#include "fcdev.h"




#define MLX4_CMD_CONFIG_FC	0x4a
#define MLX4_CMD_SET_VLAN_FLTR	0x47

#define MLX4_CMD_MOD_FC_ENABLE	0
#define MLX4_CMD_MOD_FC_DISABLE	1

#define MLX4_CMD_INMOD_BASIC_CONF	0x0000
#define MLX4_CMD_INMOD_NPORT_TAB 	0x0100

#define QPC_SERVICE_TYPE_RFCI	9
#define QPC_SERVICE_TYPE_FCMD	4
#define QPC_SERVICE_TYPE_FEXCH	5

#define MLX4_LINK_TYPE_IB	0
#define MLX4_LINK_TYPE_ETH	1

#define MLX4_FC_DB_PER_PAGE	(PAGE_SIZE / 4)

#define MLX4_FC_NUM_NPORT_IDS	128

#define MLX4_FC_USE_SRQ		0x01000000
#define MLX4_MPT_ENABLE_INVALIDATE	(0x3 << 24)

#define MLX4_FC_BIT_NEC		0x20000000
#define MLX4_FC_BIT_DESC_OWN	0x80000000
#define MLX4_FC_RFCI_OP_SEND	0xa
#define MLX4_FC_FCMD_OP_SEND	0xd
#define MLX4_FC_BIT_INS_VLAN	0x4000
#define MLX4_FC_BIT_NO_ICRC	0x2
#define MLX4_FC_BIT_TX_COMP	0xc
#define MLX4_FC_BIT_TX_IP_CS	0x10
#define MLX4_FC_BIT_TX_TCP_CS	0x20
#define MLX4_FC_BIT_TX_FCRC_CS	0x40

#define MLX4_FC_INLINE		0x80000000
#define MLX4_FC_MEMTYPE_PAD	0x100

#define QUERY_PORT_LINK_MASK	0x80

#define MLX4_FC_CQ_ARM_CMD	0x2
#define SQ_NO_PREFETCH		(1 << 7)

#define MLX4_FC_FCMD_CQ_ENTRIES 128
#define MLX4_FC_RFCI_CQ_ENTRIES 128
#define MLX4_FC_MAX_FEXCH_PER_PORT 	128

#define MLX4_FC_MAX_FMR_PAGES	64
#define MLX4_FC_FMR_PAGE_SHIFT	9

#define MLX4_DEFAULT_FC_MTU	2112
#define MLX4_DEFAULT_VLAN_ID	-1

#define RFCI_SQ_NUM_BBS		32
#define RFCI_SQ_BB_SIZE		64	/* aligned to cacheline (wqe bug), enough for 1 ctl + 3 ds */
#define RFCI_RQ_NUM_WQES	512
#define RFCI_RQ_WQE_SIZE	DS_SIZE
#define FCMD_SQ_NUM_BBS		MLX4_FC_MAX_FEXCH_PER_PORT
#define FCMD_SQ_BB_SIZE		64	/* 1 ctl + 1 addr + 1 fcp init + 1 ds */
#define FCMD_RQ_NUM_WQES	1       /* minimum allowed 2^0 */
#define FCMD_RQ_WQE_SIZE	16      /* minimum allowed 2^0 * 16 */
#define FEXCH_SQ_NUM_BBS	1       /* minimum allowed 2^0 */
#define FEXCH_SQ_BB_SIZE	16      /* minimum allowed 2^0 * 16 */
#define FEXCH_RQ_NUM_WQES	32
#define FEXCH_RQ_WQE_SIZE	16	/* 1 ds */

#define VLAN_FLTR_SIZE		128

#define	MLX4FC_DRIVER_NAME	"mlx4_fc"
#define	MLX4FC_DRIVER_VENDOR	"Mellanox"
#define	MLX4FC_DRIVER_VERSION	"0.9" 
#define MLX4FC_DRIVER_RELDATE	"June 17, 2008"
#define MLX4FC_DRIVER_AUTHOR	"Oren Duer <oren@mellanox.co.il>"
#define MLX4FC_DRIVER_DESC	"Mellanox ConnectX HCA FCoXX driver"
#define MLX4FC_DRIVER_LICENSE	"Dual BSD/GPL"

#define FCOE_WORD_TO_BYTE  	4
#define	FCOE_OUI		0x0efc00
#define	FCOE_VER		0
#define	FCOE_FLOGI_MAC 		0x0efc00fffffeULL
#define	ETHERTYPE_FCOE		0x8906
#define FC_MAX_ERROR_CNT	5

extern int mlx4_fc_vlan_id;

#define MLX4_FC_RFCI_RX_SKB_BUFSIZE	(PAGE_SIZE - 1024)
#define MLX4_FC_FCMD_RX_SKB_BUFSIZE	(PAGE_SIZE - 1024)

#define MLX4_NR_CPUS		16

#ifdef CONFIG_MLX4_EN_PREFETCH
	#define PREFETCH(x)		prefetch(x)
#else
	#define PREFETCH(x)
#endif

#ifndef MLX4_FC_ALLOC_ORDER
#define MLX4_FC_ALLOC_ORDER	2
#endif /* MLX4_FC_ALLOC_ORDER */

#define MLX4_FC_ALLOC_SIZE	(PAGE_SIZE << MLX4_FC_ALLOC_ORDER)

#define XNOR(x,y)		(!(x) == !(y))

#define foreach_fc_port(port, fcdev) \
	for (port = 1; port <= MLX4_MAX_PORTS; ++port) \
		if (test_bit(port-1, &fcdev->fc_ports))

#define mlx4_err(mdev, format, arg...) \
	dev_err(&mdev->pdev->dev, format, ## arg)
#define mlx4_info(mdev, format, arg...) \
	dev_info(&mdev->pdev->dev, format, ## arg)
#define mlx4_warn(mdev, format, arg...) \
	dev_warn(&mdev->pdev->dev, format, ## arg)

#define MIN(a, b)		((a) < (b) ? (a) : (b))
#define MAX(a, b)		((a) > (b) ? (a) : (b))

#define MLX4_PUT(dest, source, offset)				      \
	do {							      \
		void *__d = ((char *) (dest) + (offset));	      \
		switch (sizeof(source)) {			      \
		case 1: *(u8 *) __d = (source);		       break; \
		case 2:	*(__be16 *) __d = cpu_to_be16(source); break; \
		case 4:	*(__be32 *) __d = cpu_to_be32(source); break; \
		case 8:	*(__be64 *) __d = cpu_to_be64(source); break; \
		default: BUG();		      \
		}						      \
	} while (0)


#define OFFSET_IN_PAGE(v) ((u64)(v) & (PAGE_SIZE - 1))
#define SHIFT_TO_SIZE(x) (1 << (x))
#define SHIFT_TO_MASK(x) (~((u64) SHIFT_TO_SIZE(x) - 1))

#define	FCOE_ENCAPS_LEN_SOF(len, sof) \
		((FCOE_VER << 14) | (((len) & 0x3ff) << 4) | ((sof) & 0xf))
#define	FCOE_DECAPS_LEN(n)	(((n) >> 4) & 0x3ff)
#define	FCOE_DECAPS_SOF(n) \
		(((n) & 0x8) ? (((n) & 0xf) + 0x20) : (((n) & 0xf) + 0x30))

struct fcoe_hdr_old {
	net16_t		fcoe_plen;	/* fc frame len and SOF */
};

struct fcoe_crc_eof_old {
	u_int32_t	fcoe_crc32;	/* CRC for FC packet */
	net8_t		fcoe_eof;	/* EOF */
} __attribute__((packed));

enum mlx4_fcmd_io_dir {
	FCMD_IO_DIR_TARGET	= 0,
	FCMD_IO_DIR_READ,
	FCMD_IO_DIR_WRITE,
	FCMD_IO_DIR_BIDI,
};

enum mlx4_rfci_type {
	RFCI_FLOGI	= 0,
	RFCI_CTRL,
	RFCI_DATA,
	RFCI_NUM	/* must be last */
};

struct mlx4_fc_basic_config_params {
	__be32	fexch_base;
	u8	nm, nv, np;
	__be32 	fexch_base_mpt;
	u8 	log_num_rfci;
	__be32	rfci_base;
	__be32	def_fcoe_promisc_qpn;
	__be32	def_fcoe_mcast_qpn;
};

struct mlx4_query_port_context {
	u8 supported_port_type;
	u8 actual_port_type;
	__be16 mtu;
	u32 reserved2[3];
	__be64 mac;
};

struct mlx4_set_vlan_fltr_mbox {
	__be32 entry[VLAN_FLTR_SIZE];
};

struct mlx4_fc_fexch_cqe {
	__be32			my_qpn;
	__be32			invalidate_key;
	__be32			seq_id__rqpn_srq;
	__be32			xmit_byte_count;
	__be32			rcv_byte_count;
	__be32			byte_cnt;
	__be16			wqe_index;
	__be16			seq_count;
	u8			reserved[3];
	u8			owner_sr_opcode;
};

enum mlx4_en_link_state {
	LINK_DOWN,
	LINK_UP
};

struct mlx4_fc_priv;
typedef void (*comp_fn)(struct mlx4_fc_priv *, struct mlx4_cqe *);

struct mlx4_fc_cq {
	struct mlx4_cq          mcq;
	struct mlx4_hwq_resources wqres;
	int 			size;
	int 			buf_size;
	struct mlx4_fc_cqe 	*buf;
	int 			size_mask;

	char 			name[10];
	struct mlx4_fc_priv	*priv;
	comp_fn 		comp_rx, comp_tx;
};

struct mlx4_fc_info {
	void *buf;
};

struct mlx4_fc_q {
	u32 size; 
	u32 size_mask;
	u16 stride;
	u32 prod;
	u32 cons;
	void *buf;
	
	struct mlx4_fc_info *info;
};

struct mlx4_fc_qp {
	struct mlx4_qp		mqp;
	u32			buf_size;
	struct mlx4_hwq_resources wqres;
	
	struct mlx4_fc_q	sq;
	struct mlx4_fc_q	rq;
	u32 			doorbell_qpn;
};

struct mlx4_fc_rfci {
	struct mlx4_fc_qp	fc_qp;
	struct mlx4_fc_cq	fc_cq;

	uint8_t    		mac[ETH_ALEN];
	int			fc_mac_idx;
	spinlock_t		lock;
};

struct mlx4_fc_fcmd {
	struct mlx4_fc_qp	fc_qp;
	struct mlx4_fc_cq	fc_cq;
	spinlock_t		lock;
};

struct mlx4_fc_fexch {
	struct mlx4_fc_qp	fc_qp;
	struct mlx4_fmr 	fmr;
	char                    *bouncebuff;
	int			bounce_off;
	struct scatterlist	bounce_sg[1];
};

struct mlx4_fc_priv {
	struct mlx4_fc_dev	*mfcdev;

	u8			port;
	u16			fc_frame_size;

	int			fc_vlan_idx;
	u8			n_m;
	u8			n_v;
	
	struct mlx4_fc_rfci	rfci[RFCI_NUM];
	struct mlx4_fc_fcmd	fcmd;
	struct mlx4_fc_fexch	fexch[MLX4_FC_MAX_FEXCH_PER_PORT];
	struct mlx4_fc_bitmap	*fexch_bm;
	int			num_fexch;
	
	struct mlx4_fc_cq	fexch_cq[MLX4_NR_CPUS];

	int			base_rfci_qpn;
	int			base_fexch_qpn;
	int			base_fexch_mpt;
	
	uint8_t    		dest_addr[ETH_ALEN];
	net24_t 		my_fid;

	int			is_initialized; /* if true mlx4_fc_free_port has work to do */
	/*
	 * fcoe protocol address learning related stuff
	 */
	uint16_t  flogi_oxid;
	uint8_t   flogi_progress;
	uint8_t   address_mode;
	uint8_t	  fcoe_hlen;		/* FCoE header length (implies ver) */

	uint8_t			rfci_rx_enabled; /* don't start rfci rx before flogi */

	uint8_t			need_update_data_mac;
	struct sk_buff 		*deffered_plogi_skb;
	uint8_t 		deffered_plogi_eof;
};

struct mlx4_fc_dev {
	struct mlx4_dev	       *dev;

	struct fcdev		*fdev[MLX4_MAX_PORTS + 1];	/* from openfc (indices are 1 based) */

	u32			fc_ports;
	u8			num_fc_ports;
	int			base_rfci_qpn;
	int			base_fexch_qpn;
	int			base_fexch_mpt;
	int			num_rfci_qps;
	int			num_fexch_qps;
	int			num_fexch_qps_perport;
	int			log_num_mac;
	int			log_num_vlan;
	int			log_num_prio;
	
	struct list_head	pgdir_list;
	struct mutex		pgdir_mutex;
	void __iomem	       *uar_map;
	struct mlx4_uar		priv_uar;
	u32			priv_pdn;
	MLX4_DECLARE_DOORBELL_LOCK(uar_lock);

	struct mlx4_mr 		mr;

	struct device           *dma_device;
};

struct mlx4_fc_rcv_info {
	struct fcdev 	*fd;

	/* for fexch completions: */
	struct fc_scsi_pkt *fsp;
	u32 txcnt, rxcnt, bcnt;
	int xno;
	struct mlx4_fmr *fmr;
};

struct mlx4_fc_ctrl_seg {
	__be32 op_own;
	__be16 vlan;
	__be16 size;
	__be32 flags;
	__be32 parameter;
};
#define CTRL_SIZE	sizeof(struct mlx4_fc_ctrl_seg)

struct mlx4_fc_data_seg {
	__be32 count;
	__be32 mem_type;
	__be64 addr;
};
#define DS_SIZE		sizeof(struct mlx4_fc_data_seg)



struct mlx4_fc_rfci_tx_desc {
	struct mlx4_fc_ctrl_seg ctrl;
	struct mlx4_fc_data_seg data; /* at least one data segment */
};

struct mlx4_fc_rx_desc {
	struct mlx4_fc_data_seg data[0]; /* actual number of entries depends on rx ring stride */
};

struct mlx4_fc_eth_addr_seg {
	u8 	static_rate;
	u8 	reserved1[3];
	__be32 	reserved2;
	u8 	reserved3[2];
	u8	dmac[6];
};

struct mlx4_fc_init_seg {
	u8 	reserved1;
	u8 	pe;
	u16	reserved;
	u8 	cs_ctl;
	u8 	seq_id_tx;
	__be16 	mtu;
	u8 	remote_fid[3];
	u8	flags;
	__be16	remote_exch;
	__be16	local_exch_idx;
};

struct mlx4_fc_fcmd_tx_desc {
	struct mlx4_fc_ctrl_seg 	ctrl;
	struct mlx4_fc_eth_addr_seg 	addr;
	struct mlx4_fc_init_seg 	init;
	struct mlx4_fc_data_seg	data;
};

struct nport_id_entry {
	u8 	reserved;
	net24_t	fid;
};

struct mlx4_fc_rx_thread {
	int		cpu;
	struct task_struct *thread;
	struct sk_buff_head rx_list;
	struct fc_frame	frame;
};

static inline u64 mac_to_u64(u8 *mac)
{
	int i;
	u64 ret = 0;
	for (i = 0; i < 6; i++) {
		ret <<= 8;
		ret |= mac[i];
	}
	return ret;
}

static inline void u64_to_mac(u8 mac[6], u64 u64mac)
{
	int i;

	for (i = 5; i >= 0; i--) {
		mac[i] = u64mac & 0xff;
		u64mac >>= 8;
	}
}

static inline struct mlx4_fc_rcv_info *mlx4_fc_rcv_info_from_skb(struct sk_buff *skb)
{
        ASSERT(sizeof(struct mlx4_fc_rcv_info) <= sizeof(skb->cb));
        return ((struct mlx4_fc_rcv_info *) skb->cb);
}

static inline void mlx4_fc_ring_db_rx(struct mlx4_fc_qp *fc_qp)
{
	struct mlx4_fc_q *rq = &fc_qp->rq;

	wmb();
	*fc_qp->wqres.db.db = cpu_to_be32(rq->prod & 0xffff);
	wmb();
}

static inline void fc_set_mac(net8_t *mac, net24_t *fcid)
{
        mac[0] = (net8_t) (FCOE_OUI >> 16);
        mac[1] = (net8_t) (FCOE_OUI >> 8);
        mac[2] = (net8_t) FCOE_OUI;
        *(net24_t *) &mac[3] = *fcid;
}

static inline int mlx4_qp_to_reset(struct mlx4_dev *dev, struct mlx4_qp *qp)
{
	return mlx4_cmd(dev, 0, qp->qpn, 2,
			MLX4_CMD_2RST_QP, MLX4_CMD_TIME_CLASS_A);
}

/* cq */
int mlx4_fc_create_cq(struct mlx4_fc_priv *priv, struct mlx4_fc_cq *cq, int entries, int eqidx, int arm,
		      comp_fn comp_rx, comp_fn comp_tx, char *name);
void mlx4_fc_destroy_cq(struct mlx4_fc_cq *cq);
void mlx4_fc_cq_clean(struct mlx4_fc_cq *cq);

/* rx */
int mlx4_fc_post_rx_buf(struct mlx4_fc_dev *mfcdev, struct mlx4_fc_qp *fc_qp, int size, struct fc_scsi_pkt *fsp);

/* port */
void init_nport_id_table(void);
int mlx4_fc_init_port(struct mlx4_fc_dev *mfcdev, int port);
void mlx4_fc_free_port(struct mlx4_fc_dev *mfcdev, int port);
int mlx4_fc_config_nport_id(struct mlx4_fc_priv *priv, int idx, net24_t nport_id);

/* rfci */
int mlx4_fc_create_rfci(struct mlx4_fc_priv *priv, struct mlx4_fc_rfci *rfci, u64 mac);
int mlx4_fc_init_rfci(struct mlx4_fc_priv *priv, struct mlx4_fc_rfci *rfci);
void mlx4_fc_destroy_rfci(struct mlx4_fc_priv *priv, struct mlx4_fc_rfci *rfci);
int mlx4_do_rfci_xmit(void *drv_priv, int channel, struct sk_buff *skb, u8 fceof);
int mlx4_start_rfci_data(struct mlx4_fc_priv *priv, u64 mac);

/* fexch */
int mlx4_fc_create_fcmd(struct mlx4_fc_priv *priv);
void mlx4_fc_destroy_fcmd(struct mlx4_fc_priv *priv);
int mlx4_fc_init_fcmd(struct mlx4_fc_priv *priv);
int mlx4_do_fcmd_send(void *priv, struct fc_scsi_pkt *fsp);
void mlx4_fc_clean_exch(void *priv, struct scsi_cmnd *cmd, int xno);
int mlx4_fc_init_rx(void);
void mlx4_fc_kill_rx(void);

/* main */
int mlx4_fc_q_init(struct mlx4_fc_q *q, u16 stride, u32 size);
void mlx4_fc_stamp_sq(struct mlx4_fc_q *sq);



#endif /* MLX4_FC_H */
