/*
 * Copyright (c) 2008 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef MLX4_FC_DBG
#define MLX4_FC_DBG

#include <linux/highmem.h>

#define DEBUG_SG 0
#define DEBUG_BOUNCE 0
#define MLX_MAGIC 0xaf
#define BOUNCESIZE 16384
#define USE_FBO 0
#undef DEBUG_CPU_BINDING
#define FC_DBG 0

#if FC_DBG

#define FCPRINT(fmt, args...) printk("FCDBG: %s:%d: " fmt, __FUNCTION__, __LINE__, ##args)
#define HEXDUMP_OFF(msg, p, off, len) do { \
	int i; \
	printk("FCDBG: %s:%d: " msg, __FUNCTION__, __LINE__); \
	for (i=(off); i < (len)+(off); i++) { \
		if (i % 4 == 0) printk("\n%05x: ", i); \
		printk("%02x ", *(((unsigned char *)p)+i)); \
	} \
	printk("\n"); \
} while(0)
#undef ASSERT
#define ASSERT(x) if (!(x)) { \
	printk("%s:%d: ASSERT(%s) failed\n",__FUNCTION__,__LINE__, #x); \
	BUG(); \
}
#define ASSERT_PR(x, fmt, args...) if (!(x)) { \
	printk("%s:%d: ASSERT(%s) failed. Info:\n",__FUNCTION__,__LINE__, #x); \
	printk(fmt, ##args); \
	BUG(); \
}

#else

#define FCPRINT(fmt, args...) do {} while (0)
#define HEXDUMP_OFF(msg, p, off, len) do {} while (0)
#undef ASSERT
#define ASSERT(x) if (!(x)) { \
	printk("%s:%d: ASSERT(%s) failed\n",__FUNCTION__,__LINE__, #x); \
}
#define ASSERT_PR(x, fmt, args...) if (!(x)) { \
	printk("%s:%d: ASSERT(%s) failed. Info:\n",__FUNCTION__,__LINE__, #x); \
	printk(fmt, ##args); \
}

#endif

#define FCPRINT_MAC(fmt, mac, args...) FCPRINT(fmt " %02x:%02x:%02x:%02x:%02x:%02x\n", ##args, mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
#define HEXDUMP(msg, p, len) HEXDUMP_OFF(msg, p, 0, len)
#define MLX4_POISON  		((void *) 0x00100500)

#endif
