/*
 * Copyright (c) 2008 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>

#include <linux/mlx4/driver.h>
#include <linux/mlx4/cmd.h>
#include <linux/mlx4/qp.h>

#include "mlx4_fc.h"
#include "fc_frame.h"

int mlx4_fc_post_rx_buf(struct mlx4_fc_dev *mfcdev, struct mlx4_fc_qp *fc_qp, int size, struct fc_scsi_pkt *fsp)
{
	struct mlx4_fc_q *rq = &fc_qp->rq;
	struct mlx4_fc_rcv_info *fr;
	struct mlx4_fc_rx_desc *rx_desc;
	struct sk_buff *skb;
	dma_addr_t dma;
	int index = rq->prod & rq->size_mask;

	if ((u32) (rq->prod - rq->cons) == rq->size) {
		FCPRINT("RFCI rq is full: rq->prod: 0x%x, rq->cons: 0x%x, rq->size: 0x%x\n", rq->prod, rq->cons, rq->size);
		return -ENOMEM;
	}

	skb = dev_alloc_skb(size);
	if (!skb) {
		FCPRINT("Couldn't allocate skb!\n");
		return -ENOMEM;
	}

	skb_reserve(skb, FC_FRAME_HEADROOM);
	rq->info[index].buf = skb;
	
	fr = mlx4_fc_rcv_info_from_skb(skb);
	fr->fsp = fsp;

	dma = pci_map_single(mfcdev->dev->pdev, skb->data, size, PCI_DMA_FROMDEVICE);

	rx_desc = rq->buf + (index * rq->stride);

	rx_desc->data[0].count = cpu_to_be32(size);
	rx_desc->data[0].mem_type = cpu_to_be32(mfcdev->mr.key);
	rx_desc->data[0].addr = cpu_to_be64(dma);

	++rq->prod;

	return 0;
}

