/*
 * Copyright(c) 2007 Intel Corporation. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * Maintained at www.Open-FCoE.org
 */

#include "sa_kernel.h"

#include <linux/timer.h>
#include <linux/jiffies.h>

#include "sa_assert.h"
#include "ofc_dbg.h"
#include "sa_timer.h"

/*
 * Allocate a timer structure.  Set handler.
 */
struct sa_timer *sa_timer_alloc(void (*handler) (void *), void *arg)
{
	struct sa_timer *tm;

	tm = sa_malloc(sizeof(*tm));
	if (tm)
		sa_timer_init(tm, handler, arg);
	return tm;
}

u_int64_t sa_timer_get(void)
{
	return (u_int64_t) get_jiffies_64() * TICK_NSEC;
}

/*
 * Get monotonic time since some arbitrary time in the past.
 * If _POSIX_MONOTONIC_CLOCK isn't available, we'll use time of day.
 */
u_int sa_timer_get_secs(void)
{
	return jiffies_to_msecs(get_jiffies_64()) / 1000;
}

/*
 * Set timer to fire.   Delta is in microseconds from now.
 */
void sa_timer_set(struct sa_timer *tm, u_long delta_usec)
{
	mod_timer((struct timer_list *)tm,
		  jiffies + usecs_to_jiffies(delta_usec));
}

/*
 * Cancel timer if it is active.
 */
void sa_timer_cancel(struct sa_timer *tm)
{
	del_timer((struct timer_list *)tm);
}

/*
 * Free (and cancel) timer.
 */
void sa_timer_free(struct sa_timer *tm)
{
	del_timer_sync((struct timer_list *)tm);
	sa_free(tm);
}
