#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/uio.h>

#define BUFSIZE (4*4096)
#define BUF1_OFFSET 512
#define BUF2_OFFSET (2*4096+2*512)
#define BUF1_LEN 512
#define BUF2_LEN 512
#define PATTERN_CHAR 0x35
#define DATA_CHAR 0

#define ALIGN_512(p) do { p = (char *)((unsigned long)p & ~((unsigned long)0x200-1)); } while (0)

void hexdump(const char *msg, char *b, int len)
{
	int i;

	printf("%s", msg);
	for (i=0; i<len; ++i) {
		if (i % 16 == 0)
			printf("\n%08x:", i);
		if (i % 8 == 0)
			printf(" ");
		printf("%02hhx ", b[i]);
	}
	printf("\n");
}

int main(int argc, char *argv[])
{
	int fd;
	char *buf, *buf1, *buf2, *p;
	struct iovec vec[2];
	ssize_t r;
	char expected;
	
        if (argc != 2) {
		fprintf(stderr, "Usage: dread <file>\n");
		return 1;
	}

	printf("Reading file: %s\n", argv[1]);

	fd = open(argv[1], O_RDONLY | O_DIRECT);
	if (fd < 0) {
		perror("Could not open file\n");
		return 1;
	}

	buf = malloc(BUFSIZE);
	if (!buf) {
		fprintf(stderr,"Could not alloc 4*4K for buf1\n");
		return 1;
	}
	printf("buf= %p\n", buf);
	memset(buf, PATTERN_CHAR, BUFSIZE);

	buf1 = buf + BUF1_OFFSET;
	ALIGN_512(buf1);
	printf("buf1=%p\n", buf1);
	
	buf2 = buf + BUF2_OFFSET;
	ALIGN_512(buf2);
	printf("buf2=%p\n", buf2);

	vec[0].iov_base = buf1;
	vec[0].iov_len = BUF1_LEN;
	vec[1].iov_base = buf2;
	vec[1].iov_len = BUF2_LEN;

	r = readv(fd, vec, 2);
	if (r < 0) {
		perror("Error in readv");
		return 1;
	}
	if (r != (BUF1_LEN + BUF2_LEN)) {
		fprintf(stderr, "was reading %d, expected %d\n", r, (BUF1_LEN + BUF2_LEN));
		return 1;
	}

	expected = PATTERN_CHAR;
	for (p = buf; p < (buf + BUFSIZE); ++p) {
		if ( (p == buf1) || (p == buf2) )		
			expected = DATA_CHAR;
		if ( (p == buf1 + BUF1_LEN) || (p == buf2 + BUF2_LEN) )
			expected = PATTERN_CHAR;
		if (*p != expected) {
			printf("At offset 0x%x: expected 0x%x, found 0x%x.\n"
				"buf1 offset 0x%x, len 0x%x\n"
				"buf2 offset 0x%x, len 0x%x\n",
				(p - buf), expected, *p,
				(buf1 - buf), BUF1_LEN,
				(buf2 - buf), BUF2_LEN);
			hexdump("Entire buff:", buf, BUFSIZE);
			return 1;
		}
	}		
	printf("OK\n");
	return 0;
}
