#!/bin/bash
 #
 # Copyright (c) 2007 Mellanox Technologies. All rights reserved.
 #
 # This software is available to you under a choice of one of two
 # licenses.  You may choose to be licensed under the terms of the GNU
 # General Public License (GPL) Version 2, available from the file
 # COPYING in the main directory of this source tree, or the
 # OpenIB.org BSD license below:
 #
 #     Redistribution and use in source and binary forms, with or
 #     without modification, are permitted provided that the following
 #     conditions are met:
 #
 #      - Redistributions of source code must retain the above
 #        copyright notice, this list of conditions and the following
 #        disclaimer.
 #
 #      - Redistributions in binary form must reproduce the above
 #        copyright notice, this list of conditions and the following
 #        disclaimer in the documentation and/or other materials
 #        provided with the distribution.
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 # EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 # MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 # NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 # BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.

#Check that we are root to be able to change system parameters 
if [ $UID != 0 ]; then
  echo Must be root to run this script.
  exit 1
fi
# Added the typical path for SLES10 and RH5.0 
PATH=$PATH:/sbin:/usr/bin:/usr/sbin
set_affinity=""
#ifconfig="`which ifconfig`"
#sysctlcmd="`which sysctl`"
#setpci="`which setpci`"
#ethtoolcmd="`which ethtool`" 
sysctls=(                                                                 
    "net.ipv4.tcp_timestamps=0"
    "net.ipv4.tcp_sack=0"
    "net.core.netdev_max_backlog=250000"
    "net.core.rmem_max=16777216"
    "net.core.wmem_max=16777216"
    "net.core.rmem_default=16777216"
    "net.core.wmem_default=16777216"
    "net.core.optmem_max=16777216"
)
# Check the exsistanec  of utility on the system
# return the full path or break on error
Pathto()
{
local check="`which $1`"
if [ $check == "" ] ; then
    echo_failure "Can't find util : $1"
    return 1
else
    echo $1
fi
}
ethtoolcmd=$(Pathto ethtool)
ifconfig=$(Pathto ifconfig)
sysctlcmd=$(Pathto sysctl)
setpci=$(Pathto setpci)
###for reporting
echo_success() {
  echo -n $@
  [ "$BOOTUP" = "color" ] && $MOVE_TO_COL
  echo -n "[  "
  [ "$BOOTUP" = "color" ] && $SETCOLOR_SUCCESS
  echo -n $"OK"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  echo -n "  ]"
  echo -e "\r"
  return 0
}
echo_failure() {
  echo -n $@
  [ "$BOOTUP" = "color" ] && $MOVE_TO_COL
  echo -n "["
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo -n $"FAILED"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  echo -n "]"
  echo -e "\r"
  return 1
}
echo_warn() {
  echo -n $@
  [ "$BOOTUP" = "color" ] && $MOVE_TO_COL
  echo -n "["
  [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
  echo -n $"WARNING"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  echo -n "]"
  echo -e "\r"
  return 1
}
#check PCIe link details
check_pcie_link()
{
for link in `$setpci -d "15b3:" 72 | cut -b1 ` ; do
    if [ "$link" != "8" ]; then
	echo_failure "Link width is not x8"
    else
	echo_success "PCIe link is x8"
    fi
done
for link in `$setpci -d "15b3:" 72 | cut -b2` ; do
 if [ "$link" == "1" ]; then
    	echo_success "PCIe Speed 2.5Gb/s"
    elif [ "$link" == "2" ] ; then
        echo_success "PCIe Speed 5Gb/s"
    else 
	echo_warn "Warning PCIe speed unknown."
    fi
done

for maxread in `$setpci -d "15b3:" 68.w | cut -b1` ; do
 if [ "$maxread" -lt 2 ]; then
	echo_failure "MaxReadReq size is smaller than 512"
    else
	echo_success "MaxReadReq size configured"
    fi
done
}
#Apply sysctl settings
config_sys_ctrl()
{

### IPV4 specific settings
# turns TCP timestamp support off, default 1, reduces CPU use
# turn SACK support off, default on
# Change buffer sizes.
for line in ${sysctls[@]}; do
    $sysctlcmd -w $line
done
echo "16777216 16777216 16777216" > /proc/sys/net/ipv4/tcp_mem
echo "4096 87380 16777216" > /proc/sys/net/ipv4/tcp_rmem
echo "4096 65536 16777216" > /proc/sys/net/ipv4/tcp_wmem
# print to screen as well.
$sysctlcmd net.ipv4.tcp_mem
$sysctlcmd net.ipv4.tcp_rmem
$sysctlcmd net.ipv4.tcp_wmem
}

#Check if irq balance is running and turn it off.
check_disable_irqdaemon()
{
for irqb in /etc/init.d/irq* ; do
    break
done
if [ "$set_affinity" == "" ]; then
    if $irqb status 1 >/dev/null 2>&1; then
	echo "Stopping irq balancer daemon."
	$irqb stop 1 >/dev/null 2>&1 
    else
	echo "Irq balancer daemon not running." 
    fi
else
    #We could not change the intterupt manually
    #lets hope that at least the irq daemon is runinng
    if $irqb status 1 >/dev/null 2>&1; then
	echo "Irq balancer daemon running."
    else
	echo "Irq balancer daemon not running trying to run it." 
	$irqb start 1 >/dev/null 2>&1 
	#lets check if it is running now
	if $irqb status 1 >/dev/null 2>&1; then
	    echo_success "Irq balancer daemon running."
	else
	    echo_failure "irq balancer daemon not running and interrupt affinity can't be set."
	fi
    fi
fi
}
# Set the Intterupts affinity
set_irq_affinity()
{
# Set the Intterupts affinity in Tx side (if driver uses Tx intterupts)
#No interrupt in TX on MP
if [ "$vec_name" !=  "mlx4_" ]; then
	irqs="`cat /proc/interrupts | grep "$mtnic" | grep tx | sed s/":.*tx"/':'/`"
	if [ "`cat /proc/interrupts | grep "$mtnic" | grep IO-API`" != "" ] ; then
	    echo_warn "Tx Interrupt is NOT MSI-X"
	fi
	#irqs="`cat /proc/interrupts | grep "$vec_name"`"
for irq in $irqs; do
	vec=`echo $irq | cut -d':' -f1`
	ring=`echo $irq | cut -d':' -f2`
	mask=`echo $ring | awk '{printf("%x", lshift(1,$1))}'`
	echo "Tx ring $ring - setting $mask > /proc/irq/$vec/smp_affinity"
        echo $mask > /proc/irq/$vec/smp_affinity
	after_vec_hex=`cat /proc/irq/$vec/smp_affinity | tail -c3 | sed -e 's:^0[bBxX]::' | tr '[a-f]' '[A-F]'`
	after_vec=`echo ""$after_vec_hex" 10 o p" | dc`
	if [ "$mask" != "$after_vec" ]; then
	    echo_warn "Warning could not set the intterupt affinity"
	    set_affinity="Warn"
	fi
done
fi
# Set the Intterupts affinity in Rx side
if [ "$vec_name" !=  "mlx4_" ]; then
	irqs="`cat /proc/interrupts | grep "$mtnic" | grep rx | sed s/":.*rx"/':'/`"
	if [ "$irqs" = "" ] ; then
   		echo_warn "Rx Interrupt is NOT MSI-X"
	fi
else
	irqs="`cat /proc/interrupts | grep "$vec_name" | grep eth | cut -d':' -f1 `"
fi
ring=-1 
for irq in $irqs; do
	vec=`echo $irq | cut -d':' -f1`
	if [ "$vec_name" !=  "mlx4_" ]; then
		ring=`echo $irq | cut -d':' -f2`
	else
		ring=$(($ring+1))	
	fi
	mask=`echo $ring | awk '{printf("%x", lshift(1,$1))}'`
	echo "Rx ring $ring - setting $mask > /proc/irq/$vec/smp_affinity"
	echo $mask > /proc/irq/$vec/smp_affinity
	after_vec_hex=`cat /proc/irq/$vec/smp_affinity | tail -c3 | sed -e 's:^0[bBxX]::' | tr '[a-f]' '[A-F]'`
	after_vec=`echo ""$after_vec_hex" 10 o p" | dc`
	if [ "$mask" != "$after_vec" ]; then
	    echo_warn "Warning could not set the intterupt affinity"
	    set_affinity="Warn"
	fi
done
}
#########################################################################
# Get a sane screen width
[ -z "${COLUMNS:-}" ] && COLUMNS=80

[ -z "${CONSOLETYPE:-}" ] && [ -x /sbin/consoletype ] && CONSOLETYPE="`/sbin/consoletype`"

if [ -f /etc/sysconfig/i18n -a -z "${NOLOCALE:-}" ] ; then
  . /etc/sysconfig/i18n
  if [ "$CONSOLETYPE" != "pty" ]; then
        case "${LANG:-}" in
                ja_JP*|ko_KR*|zh_CN*|zh_TW*)
                        export LC_MESSAGES=en_US
                        ;;
                *)
                        export LANG
                        ;;
        esac
  else
        export LANG
  fi
fi
# Read in our configuration
  if [ -f /etc/sysconfig/init ]; then
      . /etc/sysconfig/init
  else
    # This all seem confusing? Look in /etc/sysconfig/init,
    # or in /usr/doc/initscripts-*/sysconfig.txt
    BOOTUP=color
    RES_COL=60
    MOVE_TO_COL="echo -en \\033[${RES_COL}G"
    SETCOLOR_SUCCESS="echo -en \\033[1;32m"
    SETCOLOR_FAILURE="echo -en \\033[1;31m"
    SETCOLOR_WARNING="echo -en \\033[1;33m"
    SETCOLOR_NORMAL="echo -en \\033[0;39m"
    LOGLEVEL=1
  fi
  if [ "$CONSOLETYPE" = "serial" ]; then
      BOOTUP=serial
      MOVE_TO_COL=
      SETCOLOR_SUCCESS=
      SETCOLOR_FAILURE=
      SETCOLOR_WARNING=
      SETCOLOR_NORMAL=
  fi
# Check if ConnectX En driver is up and running config the intterupt affinity.
mtnic=""
vec_name=""
for i in $( $ifconfig -a | grep eth | cut -b1-5 ); do
            if [ "`$ethtoolcmd -i $i | grep driver | cut -b9-13`" == "mtnic" ] || [ "`$ethtoolcmd -i $i | grep driver | cut -b9-13`" == "mlx4_" ]; then
                echo Found mlxnet if : $i
                mtnic=$i
		if [ "`$ethtoolcmd -i $i | grep driver | cut -b9-13`" == "mlx4_" ];then
		    echo "MP device"
		    vec_name="mlx4_"
		fi		
		#need to make sure the if is up
		$ifconfig $mtnic up
                set_irq_affinity
            fi
        done
if [ "$mtnic" == "" ]; then
  echo_failure ConnectX En driver is NOT loaded. Please load driver.
  exit 2
fi
# Check and turn off the irq balancer
check_disable_irqdaemon 
# Config the stack parameters
config_sys_ctrl
#check the pcie link
check_pcie_link
